/*
 * Team Controller - the CONTROLLER in this MVC-example.
 * Code-Beispiel zum Buch Patterns Kompakt, Spektrum Akademischer Verlag
 * Copyright 2012 Karl Eilebrecht
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.calamanari.pk.modelviewcontroller;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Logger;

import javax.swing.JRadioButton;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Team Controller - the CONTROLLER in this MVC-example.
 * @author <a href="mailto:Karl.Eilebrecht(a/t)freenet.de">Karl Eilebrecht</a>
 */
public class TeamController {

	
	/**
	 * logger
	 */
	private static final Logger LOGGER = Logger.getLogger(TeamController.class.getName());
	

	/**
	 * reference to the model
	 */
	private final TeamModel model;
	
	/**
	 * reference to the view
	 */
	private final TeamView view;
	
	
	/**
	 * task to update the view
	 */
	private final Runnable updateViewTask = new Runnable() {
		public void run() {
			updateView();
		}
	};
	
	
	/**
	 * Creates new controller responsible for the given model and the given view
	 * @param teamModel
	 * @param teamView
	 */
	public TeamController(TeamModel teamModel, TeamView teamView) {

		LOGGER.fine(this.getClass().getSimpleName() + " created, holding refences to model and view");
		
		this.model = teamModel;
		this.view = teamView;
		
		view.btnClose.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				LOGGER.fine(TeamController.class.getSimpleName() + " - [Close] clicked.");
				shutDown();
			}
		});
		
		view.btnAdd.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				LOGGER.fine(TeamController.class.getSimpleName() + " - [Add] clicked.");
				String newMember = view.txtInput.getText().trim();
				view.txtInput.setText("");
				if (newMember.length() > 0) {
					LOGGER.fine("Adding member to model");
					model.add(newMember);
				}
				else {
					LOGGER.fine("Invalid input ignored");
				}
			}
		});
		
		view.btnRemove.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				LOGGER.fine(TeamController.class.getSimpleName() + " - [Remove] clicked.");
				if (view.rad1.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 1 was selected, removing.");
					model.remove(0);
				}
				else if (view.rad2.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 2 was selected, removing.");
					model.remove(1);
				}
				else if (view.rad3.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 3 was selected, removing.");
					model.remove(2);
				}
				else if (view.rad4.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 4 was selected, removing.");
					model.remove(3);
				}
				else if (view.rad5.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 5 was selected, removing.");
					model.remove(4);
				}
				else if (view.rad6.isSelected()) {
					LOGGER.fine(TeamController.class.getSimpleName() + " - entry 6 was selected, removing.");
					model.remove(5);
				}
				else {
					LOGGER.fine(TeamController.class.getSimpleName() + " - no entry selected, request ignored.");
				}
				view.radioGroup.clearSelection();
				view.txtInput.setText("");
			}
		});
		
		
		ChangeListener selectionListener = new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				LOGGER.fine(TeamController.class.getSimpleName() + " - selection changed, updating view.");
				updateView();
			}
			
		};

		view.rad1.addChangeListener(selectionListener);
		view.rad2.addChangeListener(selectionListener);
		view.rad3.addChangeListener(selectionListener);
		view.rad4.addChangeListener(selectionListener);
		view.rad5.addChangeListener(selectionListener);
		view.rad6.addChangeListener(selectionListener);
		
		LOGGER.fine("Controls from view are now under surveillance of controller");

		
		// observe the model
		model.setModelObserver(new TeamModel.TeamModelObserver() {
			@Override
			public void handleModelChanged() {
				LOGGER.fine(TeamController.class.getSimpleName() + " - model changed, updating view.");
				SwingUtilities.invokeLater(updateViewTask);
			}
		});

		LOGGER.fine("Model is now under surveillance of controller");
		
		
		// initialize view status
		SwingUtilities.invokeLater(updateViewTask);		
		
	}
	
	
	/**
	 * Updates the view according to the current state
	 */
	private void updateView() {
		
		LOGGER.fine(this.getClass().getSimpleName() + ".updateView called ...");
		

		
		String member1 = model.getMember(0);
		String member2 = model.getMember(1);
		String member3 = model.getMember(2);
		String member4 = model.getMember(3);
		String member5 = model.getMember(4);
		String member6 = model.getMember(5);
		
		view.lblMember1.setText(member1 == null ? "" : member1);
		view.lblMember2.setText(member2 == null ? "" : member2);
		view.lblMember3.setText(member3 == null ? "" : member3);
		view.lblMember4.setText(member4 == null ? "" : member4);
		view.lblMember5.setText(member5 == null ? "" : member5);
		view.lblMember6.setText(member6 == null ? "" : member6);

		LOGGER.fine("Controller has updated data on view");		
		
		// update controls' status
		updateMemberControlStatus(member1, view.rad1);
		updateMemberControlStatus(member2, view.rad2);
		updateMemberControlStatus(member3, view.rad3);
		updateMemberControlStatus(member4, view.rad4);
		updateMemberControlStatus(member5, view.rad5);
		updateMemberControlStatus(member6, view.rad6);

		
		view.btnAdd.setEnabled(member6 == null);
		view.txtInput.setEnabled(member6 == null);
		
		view.btnRemove.setEnabled(view.radioGroup.getSelection() != null);
		
		// update color of signal bar
		if (member2 == null) {
			// need more members
			view.pnlSignal.setBackground(Color.RED);
		}
		else if (member3 == null || member4 == null) {
			// more members would be better
			view.pnlSignal.setBackground(Color.YELLOW);
		}
		else  {
			// enough members
			view.pnlSignal.setBackground(Color.LIGHT_GRAY);
		}
		
		LOGGER.fine("Controller has updated the status (enabled/disabled/signal color) of controls on view");		
		
	}

	/**
	 * updates the control depending on the current status
	 * @param member
	 * @param radioButton
	 */
	private void updateMemberControlStatus(String member, JRadioButton radioButton) {
		if (member == null) {
			if (radioButton.isSelected()) {
				view.radioGroup.clearSelection();
			}
			radioButton.setEnabled(false);
		}
		else {
			radioButton.setEnabled(true);
		}
		
	}
	
	/**
	 * Close view and shutdown
	 */
	public void shutDown() {
		LOGGER.fine(this.getClass().getSimpleName() + ".shutDown() called.");
		this.view.setVisible(false);
		this.view.dispose();
	}

}
