/*
 * Example Scripting Framework
 * Code-Beispiel zum Buch Patterns Kompakt, Spektrum Akademischer Verlag
 * Copyright 2012 Karl Eilebrecht
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.calamanari.pk.plugin;

import java.io.File;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import de.calamanari.pk.util.MiscUtils;

/**
 * This minimum framework demonstrates the usage of the PLUGIN pattern. 
 * @author <a href="mailto:Karl.Eilebrecht(a/t)freenet.de">Karl Eilebrecht</a>
 */
public class ExampleScriptingFramework implements MacroPluginFramework {

	/**
	 * logger
	 */
	public static final Logger LOGGER = Logger.getLogger(ExampleScriptingFramework.class.getName());	
	
	/**
	 * simulates some kind of system registry (JNDI or whatever)
	 */
	private final Map<String, Object> properties = new ConcurrentHashMap<String, Object>();
	
	/**
	 * allows protocol messages from macro
	 */
	private final StringBuffer protocol = new StringBuffer();
	
	@Override
	public Object getProperty(String propertyName) {
		LOGGER.fine(this.getClass().getSimpleName() + ".getProperty('" + propertyName + "') called.");
		return properties.get(propertyName.toLowerCase());
	}

	@Override
	public void setProperty(String propertyName, Object value) {
		LOGGER.fine(this.getClass().getSimpleName() + ".setProperty('" + propertyName + "', '" + value + "') called.");
		properties.put(propertyName.toLowerCase(), value);
	}

	@Override
	public boolean isPropertyAvailable(String propertyName) {
		LOGGER.fine(this.getClass().getSimpleName() + ".isPropertyAvailable('" + propertyName + "') called.");
		return properties.containsKey(propertyName.toLowerCase());
	}

	@Override
	public void addProtocolMessage(String source, String message) {
		LOGGER.fine(this.getClass().getSimpleName() + ".addProtocolMessage() called.");
		protocol.append(source);
		protocol.append(": ");
		protocol.append(message);
		protocol.append('\n');
		
	}
	
	/**
	 * Returns the protocol.
	 * @return protocol as a string
	 */
	public String getProtocol() {
		return this.protocol.toString();
	}
	
	/**
	 * Executes the script from the given file using the specified factory.
	 * @param scriptFile file with script line by line
	 * @param pluginFactory factory for retrieving plugins for macro execution
	 */
	public void executeScriptFile(File scriptFile, MacroPluginFactory pluginFactory) {
		LOGGER.fine(this.getClass().getSimpleName() + ".executeScriptFile(" + scriptFile + ") called.");
		String scriptContent = MiscUtils.readFileToString(scriptFile, "ISO-8859-1");
		String[] lines = scriptContent.split("\n");
		int len = lines.length;
		for (int i = 0; i < len; i++)  {
			String line = lines[i];
			line = line.trim();
			if (line.length() > 0 && !line.startsWith("#"))  {
				LOGGER.fine("Script line: "  + line);
			}
			else {
				continue;
			}
			
			// very poor scripting implementation :-)
			
			// find variable name for result assignment
			String resultVarName = null;
			int eqPos = line.indexOf('=');
			if (eqPos > 0) {
				resultVarName = line.substring(0, eqPos).trim();
			}
			else {
				throw new RuntimeException("Error in line " + (i+1) + ": no result variable (expected x=macro(y,..))");
			}
			
			//find expression
			String macroName = null;
			String[] argumentNames = null;
			int openPos = line.indexOf('(', eqPos);
			int closePos = line.indexOf(')', eqPos);
			if (openPos > 0 && closePos > 0 && closePos > openPos) {
				macroName = line.substring(eqPos+1, openPos).trim();
				if (macroName.length() == 0)  {
					throw new RuntimeException("Error in line " + (i+1) + ": no expression (expected x=macro(y,..))");
				}
				String argumentString = line.substring(openPos+1, closePos).trim();
				if (argumentString.length() > 0) {
					argumentNames = argumentString.split(",");
				}
			}
			else {
				throw new RuntimeException("Error in line " + (i+1) + ": no expression (expected x=macro(y,..))");
			}
			
			Object[] macroArguments = null;
			if (argumentNames != null && argumentNames.length > 0) {
				int numberOfArguments = argumentNames.length;
				macroArguments = new Object[numberOfArguments];
				for (int j = 0; j < numberOfArguments; j++) {
					String argumentName = argumentNames[j].trim();
					Object arg = null;
					if (isPropertyAvailable(argumentName)) {
						arg = getProperty(argumentName);						
					}
					if (arg == null && argumentName.length() > 0) {
						if (argumentName.startsWith("\"")) {
							argumentName = argumentName.substring(1); 
						}
						if (argumentName.endsWith("\"")) {
							argumentName = argumentName.substring(0, argumentName.length()-1); 
						}
						arg = argumentName;
					}
					macroArguments[j] = arg;
				}
			}
			else {
				macroArguments = new Object[0];
			}
			
			Object result = null;
			try {
				MacroPluginRuntime plugin = pluginFactory.getPluginForMacro(macroName);
				result = plugin.executeMacro(macroName, macroArguments);
			}
			catch (Exception ex) {
				throw new RuntimeException("Unexpected Error in line " + (i+1), ex);
			}
			
			setProperty(resultVarName, result);
		}
	}
	
}
