/*
 * Sha1HashStrategy - concrete hash STRATEGY using SHA-1.
 * Code-Beispiel zum Buch Patterns Kompakt, Spektrum Akademischer Verlag
 * Copyright 2012 Karl Eilebrecht
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.calamanari.pk.strategy;

import java.security.MessageDigest;
import java.util.logging.Logger;

/**
 * Sha1HashStrategy - concrete hash STRATEGY using SHA-1.
 * @author <a href="mailto:Karl.Eilebrecht(a/t)freenet.de">Karl Eilebrecht</a>
 */
public class Sha1HashStrategy extends HashStrategy {

	/**
	 * logger
	 */
	protected static final Logger LOGGER = Logger.getLogger(Sha1HashStrategy.class.getName());
	
	
	/**
	 * name (key) of this strategy
	 */
	public static final String STRATEGY_NAME = "SHA1";

	/**
	 * Creates new hash strategy instance
	 */
	public Sha1HashStrategy() {
		super(STRATEGY_NAME);
	}
	

	@Override
	public String computeHash(String text) {
		LOGGER.fine("Concrete Strategy " + STRATEGY_NAME + " computes hash ...");
		return computeSecureHash(text);
	}
	
	
	/**
	 * Computes a secure hash (SHA-1, according to ISO/IEC 10118-3:2004)
	 * from the given string.<br>
	 * <i>For further information on SHA-1 see also:</i>
	 * <ul>
	 *   <li>http://www.itl.nist.gov/fipspubs/fip180-1.htm</li>
	 *   <li>http://java.sun.com/javase/6/docs/technotes/guides/security/crypto/CryptoSpec.html</li>
	 *   <li>http://java.sun.com/javase/6/docs/technotes/guides/security/SunProviders.html</li>
	 * </ul>
	 * @param salt base string to be initially included in all hash computations (secret part when hash is used for security checks), may be null or empty
	 * @param sourceMap to compute hash from values, if null or empty an empty String will be returned
	 * @param skipOnNullValue skip the current mapping if the value for the key is null
	 * @return hash-string (hex), never null
	 */
	private String computeSecureHash(String text) {
		String result = "";
        if (text != null && text.length() > 0) {
			try {
				MessageDigest md = MessageDigest.getInstance("SHA-1");
		        md.reset();
        		md.update(text.getBytes("UTF-8"));
		        result = convertBytesToHexString(md.digest());
			}
			catch (Exception ex) {
				throw new RuntimeException("Error applying SHA-1 hash.", ex);
			}
        }
		return result;
	}
	

	/**
	 * Converts the given bytes to a hex-string.
	 * @param hashBytes source bytes
	 * @return hex-string
	 */
	protected static String convertBytesToHexString(byte[] hashBytes) {
        StringBuilder sbResult = new StringBuilder();
        int len = hashBytes.length;
        for (int i = 0; i < len; i++) {
            sbResult.append(Integer.toHexString(0xFF & hashBytes[i]));
        }
        return sbResult.toString();
	}


}
